/*
 *  Copyright (C) 2010  Giorgio Wicklein <g.wicklein@giowisys.com>
 *
 *  This file is part of DaemonFS.
 *
 *  DaemonFS is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  DaemonFS is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Kontakter.  If not, see <http://www.gnu.org/licenses/>.
 */

//-----------------------------------------------------------------------------
// Hearders
//-----------------------------------------------------------------------------

#include <QtGui>

#include "../modules/filewatcherengine.h"
#include "mainwindow.h"
#include "startwidget.h"
#include "filewidget.h"
#include "eventswidget.h"


//-----------------------------------------------------------------------------
// Public
//-----------------------------------------------------------------------------

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent)
{
    createModules();
    createMenu();
    createCentralWidget();
    createTrayIcon();
    createConnections();

    this->setWindowTitle("DaemonFS");
    qApp->setWindowIcon(QIcon(":/icons/daemonfs.png"));
    qApp->setApplicationName(tr("DaemonFS"));
}

MainWindow::~MainWindow()
{
}



//-----------------------------------------------------------------------------
// Private
//-----------------------------------------------------------------------------

void MainWindow::createMenu()
{
    exitAction = new QAction(tr("Exit"), this);
    exitAction->setShortcut(QKeySequence::Quit);

    aboutAction = new QAction(tr("About DaemonFS"), this);
    aboutAction->setMenuRole(QAction::AboutRole);

    aboutQtAction = new QAction(tr("About Qt"), this);
    aboutQtAction->setMenuRole(QAction::AboutQtRole);

    fileMenu = new QMenu(tr("File"), this);
    helpMenu = new QMenu(tr("Help"), this);

    fileMenu->addAction(exitAction);
    helpMenu->addAction(aboutAction);
    helpMenu->addAction(aboutQtAction);

    menuBar()->addMenu(fileMenu);
    menuBar()->addMenu(helpMenu);

    connect(exitAction, SIGNAL(triggered()),
            qApp, SLOT(quit()));
    connect(aboutQtAction, SIGNAL(triggered()),
            qApp, SLOT(aboutQt()));
    connect(aboutAction, SIGNAL(triggered()),
            this, SLOT(aboutSlot()));
}

void MainWindow::createCentralWidget()
{
    stackedWidget = new QStackedWidget(this);
    startWidget = new StartWidget(this);
    fileWidget = new FileWidget(this);
    eventsWidget = new EventsWidget(this);
    eventsWidget->mainWindow = this; //temp workaround: casting fails

    stackedWidget->addWidget(startWidget);
    stackedWidget->addWidget(fileWidget);
    stackedWidget->addWidget(eventsWidget);

    setCentralWidget(stackedWidget);
}

void MainWindow::createModules()
{
    fswatch = new FileWatcherEngine(this);
}

void MainWindow::createTrayIcon()
{
    showAction = new QAction(tr("Show DaemonFS"), this);
    connect(showAction, SIGNAL(triggered()), this, SLOT(show()));
    hideAction = new QAction(tr("Hide DaemonFS"), this);
    connect(hideAction, SIGNAL(triggered()), this, SLOT(hide()));

    trayMenu = new QMenu(this);
    trayMenu->addAction(showAction);
    trayMenu->addAction(hideAction);

    //TODO set icon...
    trayIcon = new QSystemTrayIcon(this);

    trayIcon->setContextMenu(trayMenu);
    trayIcon->setIcon(QIcon(":/icons/daemonfs.png"));
    trayIcon->setToolTip(tr("DaemonFS - watching..."));

    connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
             this, SLOT(trayActivated(QSystemTrayIcon::ActivationReason)));
}

void MainWindow::createConnections()
{
    connect(startWidget, SIGNAL(monitorButtonClicked()),
            this, SLOT(monitorButtonClicked()));
    connect(startWidget, SIGNAL(aboutButtonClicked()),
            this, SLOT(aboutSlot()));
    connect(fileWidget, SIGNAL(backButtonClicked()),
            this, SLOT(backButtonClicked()));
    connect(fileWidget, SIGNAL(startButtonClicked()),
            this, SLOT(startButtonClicked()));
    connect(eventsWidget, SIGNAL(backButtonClicked()),
            this, SLOT(backButtonClicked()));
    connect(eventsWidget, SIGNAL(stopButtonClicked()),
            this, SLOT(stopButtonClicked()));
    connect(fswatch, SIGNAL(fschanged(QString)),
            this, SLOT(fschanged(QString)));
    connect(eventsWidget, SIGNAL(hideSignal()),
            this, SLOT(hide()));
    connect(eventsWidget, SIGNAL(trayMessage(QString)),
            this, SLOT(trayMessage(QString)));
}


//-----------------------------------------------------------------------------
// Private slots
//-----------------------------------------------------------------------------

void MainWindow::aboutSlot()
{
    //TODO
}

void MainWindow::monitorButtonClicked()
{
    stackedWidget->setCurrentIndex(1);
}

void MainWindow::backButtonClicked()
{
    stackedWidget->setCurrentIndex(stackedWidget->currentIndex() - 1);
}

void MainWindow::startButtonClicked()
{
    QProgressDialog progress(tr("Indexing files..."), tr("Please wait"), 0, 0, this, Qt::Sheet);
    progress.show();

    qApp->processEvents();

    QStringList a = fileWidget->getFilesList();
    fswatch->addPaths(a);
    stackedWidget->setCurrentIndex(2);
    trayIcon->show();
}

void MainWindow::stopButtonClicked()
{
    fswatch->clear();
    trayIcon->hide();
}

#include <QDebug>
void MainWindow::fschanged(QString path)
{
    eventsWidget->addEvent(path);
}

void MainWindow::trayActivated(QSystemTrayIcon::ActivationReason reason)
{
    switch (reason) {
        case QSystemTrayIcon::DoubleClick:
            if (isHidden())
                show();
            else
                hide();
            break;
        default:
            ;
     }
}

void MainWindow::trayMessage(QString message)
{
    trayIcon->showMessage(tr("DaemonFS"), message, QSystemTrayIcon::NoIcon);
}
